`timescale 1 ps / 1 ps

`include "aes_round_func.v"
`include "aes_round_fb_con.v"

module aes_round (
  input           CLK,
  input           RST,
  input           START,
  input  [127:00] DIN,
  input  [127:00] KEY,
  
  output [127:00] DOUT
);

  reg    [06:00]  count;
  reg    [03:00]  last;
  reg             ptx_ctrl;

  wire   [127:00] aes_din;
  wire   [127:00] aes_dout;

  assign DOUT = aes_dout;

  aes_round_fb_con perm_0 (.CLK(CLK), .RST(RST), .DIN0(DIN), .DIN1(aes_dout), .CTRL(ptx_ctrl), .DOUT(aes_din));

  aes_round_func func_0 (.CLK(CLK), .RST(RST), .KEY(KEY[127:96]), .DIN(aes_din[127:96]), .BCT(last), .DOUT(aes_dout[127:96]));
  aes_round_func func_1 (.CLK(CLK), .RST(RST), .KEY(KEY[95:64] ), .DIN(aes_din[095:64]), .BCT(last), .DOUT(aes_dout[95:64] ));
  aes_round_func func_2 (.CLK(CLK), .RST(RST), .KEY(KEY[63:32] ), .DIN(aes_din[063:32]), .BCT(last), .DOUT(aes_dout[63:32] ));
  aes_round_func func_3 (.CLK(CLK), .RST(RST), .KEY(KEY[31:00] ), .DIN(aes_din[031:00]), .BCT(last), .DOUT(aes_dout[31:00] ));
 
  initial
  begin
    last = 0; count = 0; ptx_ctrl = 1;  
  end

  // very basic and simple control logic (this takes care  of the first 8 128 bit input blocks)
  always @(posedge CLK)
  begin
    if (RST)
    begin
      count        <= 0;
      ptx_ctrl     <= 1;
      last         <= 4'b0000;
    end  
    else
    begin

      // shift register for last round control
      last[2:0]   <= last[3:1];
      
      if (START)
        count <= count + 1;
      
      // switch to feedback input after first round
      if (count == 7'd8)  
        ptx_ctrl  <= 0;
      
      // last round
      if (count == 7'd72)
        last[3]   <= 1'b1; 
    end
  end

endmodule    