`timescale 1 ps / 1 ps

module aes_round_fb_con (
  input           CLK,
  input           RST,
  input  [127:00] DIN0, // plaintext
  input  [127:00] DIN1, // feedback
  input           CTRL, // chose between the two
  
  output [127:00] DOUT  // this is the input to {c0,c1,c2,c3}
);
  
  wire   [127:00] din;
  
  // input 8-bit shift registers; 1 for c1, 2 for c2, 3 for c3 (non for c0)
  reg    [07:00]  e0_c1, e1_c1, e2_c1, e3_c1;
  reg    [15:00]  e0_c2, e1_c2, e2_c2, e3_c2; 
  reg    [23:00]  e0_c3, e1_c3, e2_c3, e3_c3;
  
  // choose between plaintext or feedback
  assign din = CTRL ? DIN0 : DIN1;
  
  // for simulation
  initial
  begin
    e0_c1 = 0; e1_c1 = 0; e2_c1 = 0; e3_c1 = 0;
    e0_c2 = 0; e1_c2 = 0; e2_c2 = 0; e3_c2 = 0;
    e0_c3 = 0; e1_c3 = 0; e2_c3 = 0; e3_c3 = 0;   
  end
  
  always @(posedge CLK)
  begin
    e0_c1 <= din[87:80];
    e1_c1 <= din[55:48];
    e2_c1 <= din[23:16];
    e3_c1 <= din[119:112];
    
    e0_c2 <= {e0_c2[07:00],din[47:40]  };
    e1_c2 <= {e1_c2[07:00],din[15:08]  };
    e2_c2 <= {e2_c2[07:00],din[111:104]};
    e3_c2 <= {e3_c2[07:00],din[79:72]  };
    
    e0_c3 <= {e0_c3[15:00],din[07:00]  };
    e1_c3 <= {e1_c3[15:00],din[103:96] };
    e2_c3 <= {e2_c3[15:00],din[71:64]  };
    e3_c3 <= {e3_c3[15:00],din[39:32]  };  
  end

  assign DOUT[127:96] = {din[127:120], e0_c1[07:00], e0_c2[15:08], e0_c3[23:16]};
  assign DOUT[095:64] = {din[95:88],   e1_c1[07:00], e1_c2[15:08], e1_c3[23:16]};
  assign DOUT[063:32] = {din[63:56],   e2_c1[07:00], e2_c2[15:08], e2_c3[23:16]};
  assign DOUT[031:00] = {din[31:24],   e3_c1[07:00], e3_c2[15:08], e3_c3[23:16]};  

endmodule

