
`include "aes_unrolled_func.v"
`include "aes_unrolled_perm.v"

module aes_unrolled (
  input           CLK,
  input           RST,
  input  [127:00] DIN,
  
  output [127:00] DOUT
);

  // keys are set to static values; [0..9] correspond to "round key" values 
  parameter KEY0 = 128'hA0FAFE1788542CB123A339392A6C7605;
  parameter KEY1 = 128'hF2C295F27A96B9435935807A7359F67F;         
  parameter KEY2 = 128'h3D80477D4716FE3E1E237E446D7A883B;
  parameter KEY3 = 128'hEF44A541A8525B7FB671253BDB0BAD00;
  parameter KEY4 = 128'hD4D1C6F87C839D87CAF2B8BC11F915BC;
  parameter KEY5 = 128'h6D88A37A110B3EFDDBF98641CA0093FD;
  parameter KEY6 = 128'h4E54F70E5F5FC9F384A64FB24EA6DC4F;
  parameter KEY7 = 128'hEAD27321B58DBAD2312BF5607F8D292F;
  parameter KEY8 = 128'hAC7766F319FADC2128D12941575C006E;
  parameter KEY9 = 128'hD014F9A8C9EE2589E13F0CC8B6630CA6;

  reg    [127:00] aes_din;

  wire   [127:00] r0_dout, r1_dout, r2_dout, r3_dout, r4_dout, r5_dout, r6_dout, r7_dout, r8_dout, r9_dout;  
  wire   [127:00] r0_din, r1_din, r2_din, r3_din, r4_din, r5_din, r6_din, r7_din, r8_din, r9_din;  

  assign DOUT = r9_dout;

  always @(posedge CLK)
  begin
    aes_din <= DIN;
  end
  
  // round 0
  aes_unrolled_perm perm_0 (.CLK(CLK), .RST(RST), .DIN(aes_din), .DOUT(r0_din));
                 
  aes_unrolled_func func_r0_0 (.CLK(CLK), .RST(RST), .KEY(KEY0[127:096]), .DIN(r0_din[127:96]), .BCT(4'd0), .DOUT(r0_dout[127:096]));
  aes_unrolled_func func_r0_1 (.CLK(CLK), .RST(RST), .KEY(KEY0[095:064]), .DIN(r0_din[95:64] ), .BCT(4'd0), .DOUT(r0_dout[095:064]));
  aes_unrolled_func func_r0_2 (.CLK(CLK), .RST(RST), .KEY(KEY0[063:032]), .DIN(r0_din[63:32] ), .BCT(4'd0), .DOUT(r0_dout[063:032]));
  aes_unrolled_func func_r0_3 (.CLK(CLK), .RST(RST), .KEY(KEY0[031:000]), .DIN(r0_din[31:00] ), .BCT(4'd0), .DOUT(r0_dout[031:000]));

  // round 1  
  aes_unrolled_perm perm_1 (.CLK(CLK), .RST(RST), .DIN(r0_dout), .DOUT(r1_din));
  
  aes_unrolled_func func_r1_0 (.CLK(CLK), .RST(RST), .KEY(KEY1[127:096]), .DIN(r1_din[127:96]), .BCT(4'd0), .DOUT(r1_dout[127:096]));
  aes_unrolled_func func_r1_1 (.CLK(CLK), .RST(RST), .KEY(KEY1[095:064]), .DIN(r1_din[95:64] ), .BCT(4'd0), .DOUT(r1_dout[095:064]));
  aes_unrolled_func func_r1_2 (.CLK(CLK), .RST(RST), .KEY(KEY1[063:032]), .DIN(r1_din[63:32] ), .BCT(4'd0), .DOUT(r1_dout[063:032]));
  aes_unrolled_func func_r1_3 (.CLK(CLK), .RST(RST), .KEY(KEY1[031:000]), .DIN(r1_din[31:00] ), .BCT(4'd0), .DOUT(r1_dout[031:000]));

  // round 2  
  aes_unrolled_perm perm_2 (.CLK(CLK), .RST(RST), .DIN(r1_dout), .DOUT(r2_din));
  
  aes_unrolled_func func_r2_0 (.CLK(CLK), .RST(RST), .KEY(KEY2[127:096]), .DIN(r2_din[127:96]), .BCT(4'd0), .DOUT(r2_dout[127:096]));
  aes_unrolled_func func_r2_1 (.CLK(CLK), .RST(RST), .KEY(KEY2[095:064]), .DIN(r2_din[95:64] ), .BCT(4'd0), .DOUT(r2_dout[095:064]));
  aes_unrolled_func func_r2_2 (.CLK(CLK), .RST(RST), .KEY(KEY2[063:032]), .DIN(r2_din[63:32] ), .BCT(4'd0), .DOUT(r2_dout[063:032]));
  aes_unrolled_func func_r2_3 (.CLK(CLK), .RST(RST), .KEY(KEY2[031:000]), .DIN(r2_din[31:00] ), .BCT(4'd0), .DOUT(r2_dout[031:000]));

  // round 3  
  aes_unrolled_perm perm_3 (.CLK(CLK), .RST(RST), .DIN(r2_dout), .DOUT(r3_din));
  
  aes_unrolled_func func_r3_0 (.CLK(CLK), .RST(RST), .KEY(KEY3[127:096]), .DIN(r3_din[127:96]), .BCT(4'd0), .DOUT(r3_dout[127:096]));
  aes_unrolled_func func_r3_1 (.CLK(CLK), .RST(RST), .KEY(KEY3[095:064]), .DIN(r3_din[95:64] ), .BCT(4'd0), .DOUT(r3_dout[095:064]));
  aes_unrolled_func func_r3_2 (.CLK(CLK), .RST(RST), .KEY(KEY3[063:032]), .DIN(r3_din[63:32] ), .BCT(4'd0), .DOUT(r3_dout[063:032]));
  aes_unrolled_func func_r3_3 (.CLK(CLK), .RST(RST), .KEY(KEY3[031:000]), .DIN(r3_din[31:00] ), .BCT(4'd0), .DOUT(r3_dout[031:000]));

  // round 4  
  aes_unrolled_perm perm_4 (.CLK(CLK), .RST(RST), .DIN(r3_dout), .DOUT(r4_din));
  
  aes_unrolled_func func_r4_0 (.CLK(CLK), .RST(RST), .KEY(KEY4[127:096]), .DIN(r4_din[127:96]), .BCT(4'd0), .DOUT(r4_dout[127:096]));
  aes_unrolled_func func_r4_1 (.CLK(CLK), .RST(RST), .KEY(KEY4[095:064]), .DIN(r4_din[95:64] ), .BCT(4'd0), .DOUT(r4_dout[095:064]));
  aes_unrolled_func func_r4_2 (.CLK(CLK), .RST(RST), .KEY(KEY4[063:032]), .DIN(r4_din[63:32] ), .BCT(4'd0), .DOUT(r4_dout[063:032]));
  aes_unrolled_func func_r4_3 (.CLK(CLK), .RST(RST), .KEY(KEY4[031:000]), .DIN(r4_din[31:00] ), .BCT(4'd0), .DOUT(r4_dout[031:000]));

  // round 5  
  aes_unrolled_perm perm_5 (.CLK(CLK), .RST(RST), .DIN(r4_dout), .DOUT(r5_din));
  
  aes_unrolled_func func_r5_0 (.CLK(CLK), .RST(RST), .KEY(KEY5[127:096]), .DIN(r5_din[127:96]), .BCT(4'd0), .DOUT(r5_dout[127:096]));
  aes_unrolled_func func_r5_1 (.CLK(CLK), .RST(RST), .KEY(KEY5[095:064]), .DIN(r5_din[95:64] ), .BCT(4'd0), .DOUT(r5_dout[095:064]));
  aes_unrolled_func func_r5_2 (.CLK(CLK), .RST(RST), .KEY(KEY5[063:032]), .DIN(r5_din[63:32] ), .BCT(4'd0), .DOUT(r5_dout[063:032]));
  aes_unrolled_func func_r5_3 (.CLK(CLK), .RST(RST), .KEY(KEY5[031:000]), .DIN(r5_din[31:00] ), .BCT(4'd0), .DOUT(r5_dout[031:000]));

  // round 6                                                                                                               
  aes_unrolled_perm perm_6 (.CLK(CLK), .RST(RST), .DIN(r5_dout), .DOUT(r6_din));
                                                                                                                       
  aes_unrolled_func func_r6_0 (.CLK(CLK), .RST(RST), .KEY(KEY6[127:096]), .DIN(r6_din[127:96]), .BCT(4'd0), .DOUT(r6_dout[127:096])); 
  aes_unrolled_func func_r6_1 (.CLK(CLK), .RST(RST), .KEY(KEY6[095:064]), .DIN(r6_din[95:64] ), .BCT(4'd0), .DOUT(r6_dout[095:064])); 
  aes_unrolled_func func_r6_2 (.CLK(CLK), .RST(RST), .KEY(KEY6[063:032]), .DIN(r6_din[63:32] ), .BCT(4'd0), .DOUT(r6_dout[063:032])); 
  aes_unrolled_func func_r6_3 (.CLK(CLK), .RST(RST), .KEY(KEY6[031:000]), .DIN(r6_din[31:00] ), .BCT(4'd0), .DOUT(r6_dout[031:000])); 

  // round 7                                                                                                              
  aes_unrolled_perm perm_7 (.CLK(CLK), .RST(RST), .DIN(r6_dout), .DOUT(r7_din));
                                                                                                                    
  aes_unrolled_func func_r7_0 (.CLK(CLK), .RST(RST), .KEY(KEY7[127:096]), .DIN(r7_din[127:96]), .BCT(4'd0), .DOUT(r7_dout[127:096]));
  aes_unrolled_func func_r7_1 (.CLK(CLK), .RST(RST), .KEY(KEY7[095:064]), .DIN(r7_din[95:64] ), .BCT(4'd0), .DOUT(r7_dout[095:064]));
  aes_unrolled_func func_r7_2 (.CLK(CLK), .RST(RST), .KEY(KEY7[063:032]), .DIN(r7_din[63:32] ), .BCT(4'd0), .DOUT(r7_dout[063:032]));
  aes_unrolled_func func_r7_3 (.CLK(CLK), .RST(RST), .KEY(KEY7[031:000]), .DIN(r7_din[31:00] ), .BCT(4'd0), .DOUT(r7_dout[031:000]));
                 
  // round 8                                                                                                                                
  aes_unrolled_perm perm_8 (.CLK(CLK), .RST(RST), .DIN(r7_dout), .DOUT(r8_din));
                                                                                                                     
  aes_unrolled_func func_r8_0 (.CLK(CLK), .RST(RST), .KEY(KEY8[127:096]), .DIN(r8_din[127:96]), .BCT(4'd0), .DOUT(r8_dout[127:096])); 
  aes_unrolled_func func_r8_1 (.CLK(CLK), .RST(RST), .KEY(KEY8[095:064]), .DIN(r8_din[95:64] ), .BCT(4'd0), .DOUT(r8_dout[095:064])); 
  aes_unrolled_func func_r8_2 (.CLK(CLK), .RST(RST), .KEY(KEY8[063:032]), .DIN(r8_din[63:32] ), .BCT(4'd0), .DOUT(r8_dout[063:032])); 
  aes_unrolled_func func_r8_3 (.CLK(CLK), .RST(RST), .KEY(KEY8[031:000]), .DIN(r8_din[31:00] ), .BCT(4'd0), .DOUT(r8_dout[031:000])); 

  // round 9: notice that for this round BCT=F to indicate that this is the final round                                                                                                                  
  aes_unrolled_perm perm_9 (.CLK(CLK), .RST(RST), .DIN(r8_dout), .DOUT(r9_din));
                                                
  aes_unrolled_func func_r9_0 (.CLK(CLK), .RST(RST), .KEY(KEY9[127:096]), .DIN(r9_din[127:96]), .BCT(4'hF), .DOUT(r9_dout[127:096]));  
  aes_unrolled_func func_r9_1 (.CLK(CLK), .RST(RST), .KEY(KEY9[095:064]), .DIN(r9_din[95:64] ), .BCT(4'hF), .DOUT(r9_dout[095:064]));  
  aes_unrolled_func func_r9_2 (.CLK(CLK), .RST(RST), .KEY(KEY9[063:032]), .DIN(r9_din[63:32] ), .BCT(4'hF), .DOUT(r9_dout[063:032]));  
  aes_unrolled_func func_r9_3 (.CLK(CLK), .RST(RST), .KEY(KEY9[031:000]), .DIN(r9_din[31:00] ), .BCT(4'hF), .DOUT(r9_dout[031:000]));  
                                                                                                                                
endmodule

    
    
    